=== added file 'drizzled/internal/getrusage.h'
--- drizzled/internal/getrusage.h	1970-01-01 00:00:00 +0000
+++ drizzled/internal/getrusage.h	2010-09-23 03:33:27 +0000
@@ -0,0 +1,33 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/* Only Linux defines getrusage's RUSAGE_THREAD */
+
+#ifndef DRIZZLED_INTERNAL_GETRUSAGE_H
+#define DRIZZLED_INTERNAL_GETRUSAGE_H
+
+#include <sys/time.h>
+#include <sys/resource.h>
+
+#ifndef RUSAGE_THREAD
+#define RUSAGE_THREAD RUSAGE_SELF
+#endif
+
+#endif /* DRIZZLED_INTERNAL_GETRUSAGE_H */

=== modified file 'drizzled/internal/include.am'
--- drizzled/internal/include.am	2010-08-07 22:01:28 +0000
+++ drizzled/internal/include.am	2010-09-23 03:33:27 +0000
@@ -19,6 +19,7 @@
 EXTRA_DIST+= drizzled/internal/CHARSET_INFO.txt
 noinst_HEADERS+= \
 		drizzled/internal/aio_result.h \
+		drizzled/internal/getrusage.h \
 		drizzled/internal/iocache.h \
 		drizzled/internal/m_string.h \
 		drizzled/internal/my_bit.h \

=== modified file 'drizzled/session.cc'
--- drizzled/session.cc	2010-09-21 09:57:03 +0000
+++ drizzled/session.cc	2010-09-23 03:36:20 +0000
@@ -177,7 +177,8 @@
   cached_table(0),
   transaction_message(NULL),
   statement_message(NULL),
-  session_event_observers(NULL)
+  session_event_observers(NULL),
+  use_usage(false)
 {
   memset(process_list_info, 0, PROCESS_LIST_WIDTH);
   client->setSession(this);
@@ -392,6 +393,12 @@
   plugin::Logging::postEndDo(this);
   plugin::EventObserver::deregisterSessionEvents(*this); 
 
+  for (PropertyMap::iterator iter= life_properties.begin(); iter != life_properties.end(); iter++)
+  {
+    delete (*iter).second;
+  }
+  life_properties.clear();
+
   /* Ensure that no one is using Session */
   LOCK_delete.unlock();
 }
@@ -494,6 +501,8 @@
                                 variables.query_prealloc_size);
   transaction.xid_state.xid.null();
   transaction.xid_state.in_session=1;
+  if (use_usage)
+    resetUsage();
 }
 
 bool Session::initGlobals()

=== modified file 'drizzled/session.h'
--- drizzled/session.h	2010-09-21 09:57:03 +0000
+++ drizzled/session.h	2010-09-23 03:36:20 +0000
@@ -36,6 +36,7 @@
 #include "drizzled/query_id.h"
 #include "drizzled/named_savepoint.h"
 #include "drizzled/transaction_context.h"
+#include "drizzled/util/storable.h"
 
 #include "drizzled/my_hash.h"
 
@@ -45,6 +46,8 @@
 #include <bitset>
 #include <deque>
 
+#include "drizzled/internal/getrusage.h"
+
 #include <drizzled/security_context.h>
 #include <drizzled/open_tables_state.h>
 
@@ -314,6 +317,9 @@
 class Session : public Open_tables_state
 {
 public:
+  // Plugin storage in Session.
+  typedef boost::unordered_map<std::string, util::Storable *, util::insensitive_hash, util::insensitive_equal_to> PropertyMap;
+
   /*
     MARK_COLUMNS_NONE:  Means mark_used_colums is not set and no indicator to
                         handler of fields used is set
@@ -1560,8 +1566,19 @@
   int lock_tables(TableList *tables, uint32_t count, bool *need_reopen);
 
   Table *create_virtual_tmp_table(List<CreateField> &field_list);
-
-
+  
+  drizzled::util::Storable *getProperty(const std::string &arg)
+  {
+    return life_properties[arg];
+  }
+
+  template<class T>
+  bool setProperty(const std::string &arg, T *value)
+  {
+    life_properties[arg]= value;
+
+    return true;
+  }
 
   /**
     Return the default storage engine
@@ -1582,11 +1599,36 @@
 
   void get_xid(DRIZZLE_XID *xid); // Innodb only
 
-private:
+  TableShareInstance *getTemporaryShare(TableIdentifier::Type type_arg);
+
+private:
+  bool resetUsage()
+  {
+    if (getrusage(RUSAGE_THREAD, &usage))
+    {
+      return false;
+    }
+
+    return true;
+  }
+public:
+
+  void setUsage(bool arg)
+  {
+    use_usage= arg;
+  }
+
+  const struct rusage &getUsage()
+  {
+    return usage;
+  }
+
+private:
+  // This lives throughout the life of Session
+  bool use_usage;
+  PropertyMap life_properties;
   std::vector<TableShareInstance *> temporary_shares;
-
-public:
-  TableShareInstance *getTemporaryShare(TableIdentifier::Type type_arg);
+  struct rusage usage;
 };
 
 class Join;

=== modified file 'drizzled/util/include.am'
--- drizzled/util/include.am	2010-07-28 21:35:30 +0000
+++ drizzled/util/include.am	2010-09-20 02:11:07 +0000
@@ -17,6 +17,7 @@
 nobase_dist_include_HEADERS+= \
                               drizzled/util/convert.h \
                               drizzled/util/functors.h \
+			      drizzled/util/storable.h \
 			      drizzled/util/string.h \
                               drizzled/util/test.h \
                               drizzled/util/tokenize.h

=== added file 'drizzled/util/storable.h'
--- drizzled/util/storable.h	1970-01-01 00:00:00 +0000
+++ drizzled/util/storable.h	2010-09-20 02:11:07 +0000
@@ -0,0 +1,39 @@
+/* - mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_UTIL_STORABLE_H
+#define DRIZZLED_UTIL_STORABLE_H
+
+namespace drizzled
+{
+
+namespace util
+{
+
+class Storable {
+public:
+  virtual ~Storable()
+  { }
+};
+
+} /* namespace util */
+} /* namespace drizzled */
+
+#endif /* DRIZZLED_UTIL_STORABLE_H */

=== added directory 'plugin/performance_dictionary'
=== added file 'plugin/performance_dictionary/dictionary.cc'
--- plugin/performance_dictionary/dictionary.cc	1970-01-01 00:00:00 +0000
+++ plugin/performance_dictionary/dictionary.cc	2010-09-20 02:11:07 +0000
@@ -0,0 +1,46 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "config.h"
+#include "plugin/performance_dictionary/dictionary.h"
+
+using namespace drizzled;
+
+static int init(drizzled::module::Context &context)
+{
+  context.add(new performance_dictionary::SessionUsage);
+  context.add(new performance_dictionary::SessionUsageLogger);
+  
+  return 0;
+}
+
+DRIZZLE_DECLARE_PLUGIN
+{
+  DRIZZLE_VERSION_ID,
+  "Performance Dictionary",
+  "1.0",
+  "Brian Aker",
+  "Data Dictionary for performance related table functions.",
+  PLUGIN_LICENSE_GPL,
+  init,
+  NULL,
+  NULL
+}
+DRIZZLE_DECLARE_PLUGIN_END;

=== added file 'plugin/performance_dictionary/dictionary.h'
--- plugin/performance_dictionary/dictionary.h	1970-01-01 00:00:00 +0000
+++ plugin/performance_dictionary/dictionary.h	2010-09-20 02:11:07 +0000
@@ -0,0 +1,32 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_PERFORMANCE_DICTIONARY_DICTIONARY_H
+#define PLUGIN_PERFORMANCE_DICTIONARY_DICTIONARY_H
+
+#include <drizzled/plugin/table_function.h>
+#include <drizzled/plugin/logging.h>
+
+// Prototypes
+#include "plugin/performance_dictionary/query_usage.h"
+#include "plugin/performance_dictionary/session_usage_logger.h"
+#include "plugin/performance_dictionary/session_usage.h"
+
+#endif /* PLUGIN_PERFORMANCE_DICTIONARY_DICTIONARY_H */

=== added file 'plugin/performance_dictionary/plugin.ini'
--- plugin/performance_dictionary/plugin.ini	1970-01-01 00:00:00 +0000
+++ plugin/performance_dictionary/plugin.ini	2010-09-20 02:11:07 +0000
@@ -0,0 +1,14 @@
+[plugin]
+title=Performance Dictionary
+description=Table functions related to performance issues.
+load_by_default=no
+static=no
+sources= 
+  dictionary.cc
+  session_usage.cc
+  session_usage_logger.cc
+headers=
+  dictionary.h
+  query_usage.h
+  session_usage_logger.h
+  session_usage.h

=== added file 'plugin/performance_dictionary/query_usage.h'
--- plugin/performance_dictionary/query_usage.h	1970-01-01 00:00:00 +0000
+++ plugin/performance_dictionary/query_usage.h	2010-09-23 03:33:27 +0000
@@ -0,0 +1,113 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_PERFORMANCE_DICTIONARY_QUERY_USAGE_H
+#define PLUGIN_PERFORMANCE_DICTIONARY_QUERY_USAGE_H
+
+#include "drizzled/internal/getrusage.h"
+
+#include "drizzled/session.h"
+
+namespace performance_dictionary {
+
+#define USAGE_MAX_KEPT 5
+
+struct query_usage {
+  std::string query;
+  struct rusage start;
+  struct rusage buffer;
+
+  query_usage()
+  {
+    memset(&start, 0, sizeof(struct rusage));
+    memset(&buffer, 0, sizeof(struct rusage));
+  }
+
+  void set(const std::string &sql, const struct rusage &arg)
+  {
+    if (getrusage(RUSAGE_THREAD, &buffer))
+    {
+      memset(&start, 0, sizeof(struct rusage));
+      memset(&buffer, 0, sizeof(struct rusage));
+      return;
+    }
+    query= sql.substr(0, 512);
+    start= arg;
+
+    buffer.ru_utime.tv_sec -= start.ru_utime.tv_sec;
+    buffer.ru_utime.tv_usec -= start.ru_utime.tv_usec;
+
+    buffer.ru_stime.tv_sec -= start.ru_stime.tv_sec;
+    buffer.ru_stime.tv_usec -= start.ru_stime.tv_usec;
+
+    buffer.ru_maxrss -= start.ru_maxrss;
+    buffer.ru_ixrss -= start.ru_ixrss;
+    buffer.ru_idrss -= start.ru_idrss;
+    buffer.ru_isrss -= start.ru_isrss;
+    buffer.ru_minflt -= start.ru_minflt;
+    buffer.ru_majflt -= start.ru_majflt;
+    buffer.ru_nswap -= start.ru_nswap;
+    buffer.ru_inblock -= start.ru_inblock;
+    buffer.ru_oublock -= start.ru_oublock;
+    buffer.ru_msgsnd -= start.ru_msgsnd;
+    buffer.ru_msgrcv -= start.ru_msgrcv;
+    buffer.ru_nsignals -= start.ru_nsignals;
+    buffer.ru_nvcsw -= start.ru_nvcsw;
+    buffer.ru_nivcsw -= start.ru_nivcsw;
+  }
+
+  const struct rusage &delta(void) const
+  {
+    return buffer;
+  }
+
+  ~query_usage()
+  { }
+};
+
+typedef std::list <query_usage> Query_list;
+
+class QueryUsage : public drizzled::util::Storable {
+public:
+  Query_list query_list;
+
+  QueryUsage()
+  {
+    query_list.resize(USAGE_MAX_KEPT);
+  }
+
+  void push(const std::string &sql, const struct rusage &arg)
+  {
+    Query_list::iterator it= query_list.end();
+    it--;
+    query_list.splice(query_list.begin(), query_list, it);
+    query_list.front().set(sql, arg);
+  }
+
+  Query_list &list(void)
+  {
+    return query_list;
+  }
+};
+
+
+} /* namespace performance_dictionary */
+
+#endif /* PLUGIN_PERFORMANCE_DICTIONARY_QUERY_USAGE_H */

=== added file 'plugin/performance_dictionary/session_usage.cc'
--- plugin/performance_dictionary/session_usage.cc	1970-01-01 00:00:00 +0000
+++ plugin/performance_dictionary/session_usage.cc	2010-09-20 02:11:07 +0000
@@ -0,0 +1,141 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "config.h"
+
+#include "plugin/performance_dictionary/dictionary.h"
+
+#include <drizzled/atomics.h>
+#include <drizzled/session.h>
+
+#include <sys/resource.h>
+
+using namespace drizzled;
+using namespace std;
+
+#define FUNCTION_NAME_LEN 64
+
+performance_dictionary::SessionUsage::SessionUsage() :
+  plugin::TableFunction("DATA_DICTIONARY", "SESSION_USAGE")
+{
+  add_field("QUERY", plugin::TableFunction::STRING, FUNCTION_NAME_LEN, false);
+  add_field("USER_TIME_USED_SECONDS", plugin::TableFunction::NUMBER, false);
+  add_field("USER_TIME_USED_MICRO_SECONDS", plugin::TableFunction::NUMBER, false);
+  add_field("SYSTEM_TIME_USED_SECONDS", plugin::TableFunction::NUMBER, false);
+  add_field("SYSTEM_TIME_USED_MICRO_SECONDS", plugin::TableFunction::NUMBER, false);
+  add_field("INTEGRAL_MAX_RESIDENT_SET_SIZE", plugin::TableFunction::NUMBER, 0, false);
+  add_field("INTEGRAL_SHARED_TEXT_MEMORY_SIZE", plugin::TableFunction::NUMBER, 0, false);
+  add_field("INTEGRAL_UNSHARED_DATA_SIZE", plugin::TableFunction::NUMBER, 0, false);
+  add_field("INTEGRAL_UNSHARED_STACK_SIZE", plugin::TableFunction::NUMBER, 0, false);
+  add_field("PAGE_RECLAIMS", plugin::TableFunction::NUMBER, 0, false);
+  add_field("PAGE_FAULTS", plugin::TableFunction::NUMBER, 0, false);
+  add_field("SWAPS", plugin::TableFunction::NUMBER, 0, false);
+  add_field("BLOCK_INPUT_OPERATIONS", plugin::TableFunction::NUMBER, 0, false);
+  add_field("BLOCK_OUTPUT_OPERATIONS", plugin::TableFunction::NUMBER, 0, false);
+  add_field("MESSAGES_SENT", plugin::TableFunction::NUMBER, 0, false);
+  add_field("MESSAGES_RECEIVED", plugin::TableFunction::NUMBER, 0, false);
+  add_field("SIGNALS_RECEIVED", plugin::TableFunction::NUMBER, 0, false);
+  add_field("VOLUNTARY_CONTEXT_SWITCHES", plugin::TableFunction::NUMBER, 0, false);
+  add_field("INVOLUNTARY_CONTEXT_SWITCHES", plugin::TableFunction::NUMBER, 0, false);
+}
+
+
+performance_dictionary::SessionUsage::Generator::Generator(drizzled::Field **arg) :
+  drizzled::plugin::TableFunction::Generator(arg),
+  usage_cache(0)
+{ 
+  usage_cache= dynamic_cast<QueryUsage *>(getSession().getProperty("query_usage"));
+  if (usage_cache)
+    query_iter= usage_cache->list().rbegin();
+}
+
+bool performance_dictionary::SessionUsage::Generator::populate()
+{
+  if (not usage_cache)
+    return false;
+
+  if (query_iter ==  usage_cache->list().rend())
+    return false;
+
+  publish((*query_iter).query, (*query_iter).delta());
+  query_iter++;
+
+  return true;
+}
+
+void performance_dictionary::SessionUsage::Generator::publish(const std::string &sql, const struct rusage &usage_arg)
+{
+  /* SQL */
+  push(sql);
+
+  /* USER_TIME_USED_SECONDS */
+  push(static_cast<int64_t>(usage_arg.ru_utime.tv_sec));
+
+  /* USER_TIME_USED_MICRO_SECONDS */
+  push(static_cast<int64_t>(usage_arg.ru_utime.tv_usec));
+
+  /* SYSTEM_TIME_USED_SECONDS */
+  push(static_cast<int64_t>(usage_arg.ru_stime.tv_sec));
+
+  /* SYSTEM_TIME_USED_MICRO_SECONDS */
+  push(static_cast<int64_t>(usage_arg.ru_stime.tv_usec));
+
+  /* INTEGRAL_MAX_RESIDENT_SET_SIZE */
+  push(static_cast<int64_t>(usage_arg.ru_maxrss));
+
+  /* INTEGRAL_SHARED_TEXT_MEMORY_SIZE */
+  push(static_cast<int64_t>(usage_arg.ru_ixrss));
+
+  /* INTEGRAL_UNSHARED_DATA_SIZE */
+  push(static_cast<int64_t>(usage_arg.ru_idrss));
+
+  /* INTEGRAL_UNSHARED_STACK_SIZE */
+  push(static_cast<int64_t>(usage_arg.ru_isrss));
+
+  /* PAGE_RECLAIMS */
+  push(static_cast<int64_t>(usage_arg.ru_minflt));
+
+  /* PAGE_FAULTS */
+  push(static_cast<int64_t>(usage_arg.ru_majflt));
+
+  /* SWAPS */
+  push(static_cast<int64_t>(usage_arg.ru_nswap));
+
+  /* BLOCK_INPUT_OPERATIONS */
+  push(static_cast<int64_t>(usage_arg.ru_inblock));
+
+  /* BLOCK_OUTPUT_OPERATIONS */
+  push(static_cast<int64_t>(usage_arg.ru_oublock));
+
+  /* MESSAGES_SENT */
+  push(static_cast<int64_t>(usage_arg.ru_msgsnd));
+
+  /* MESSAGES_RECEIVED */
+  push(static_cast<int64_t>(usage_arg.ru_msgrcv));
+
+  /* SIGNALS_RECEIVED */
+  push(static_cast<int64_t>(usage_arg.ru_nsignals));
+
+  /* VOLUNTARY_CONTEXT_SWITCHES */
+  push(static_cast<int64_t>(usage_arg.ru_nvcsw));
+
+  /* INVOLUNTARY_CONTEXT_SWITCHES */
+  push(static_cast<int64_t>(usage_arg.ru_nivcsw));
+}

=== added file 'plugin/performance_dictionary/session_usage.h'
--- plugin/performance_dictionary/session_usage.h	1970-01-01 00:00:00 +0000
+++ plugin/performance_dictionary/session_usage.h	2010-09-23 19:03:30 +0000
@@ -0,0 +1,53 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_PERFORMANCE_DICTIONARY_SESSION_USAGE_H
+#define PLUGIN_PERFORMANCE_DICTIONARY_SESSION_USAGE_H
+
+namespace performance_dictionary {
+
+class SessionUsage : public  drizzled::plugin::TableFunction
+{
+
+public:
+  SessionUsage();
+
+  class Generator : public drizzled::plugin::TableFunction::Generator 
+  {
+    Query_list::reverse_iterator query_iter;
+    QueryUsage *usage_cache;
+
+    void publish(const std::string &sql, const struct rusage &r_usage);
+
+  public:
+    Generator(drizzled::Field **arg);
+
+    bool populate();
+  };
+
+  Generator *generator(drizzled::Field **arg)
+  {
+    return new Generator(arg);
+  }
+};
+
+} /* namespace performance_dictionary */
+
+#endif /* PLUGIN_PERFORMANCE_DICTIONARY_SESSION_USAGE_H */

=== added file 'plugin/performance_dictionary/session_usage_logger.cc'
--- plugin/performance_dictionary/session_usage_logger.cc	1970-01-01 00:00:00 +0000
+++ plugin/performance_dictionary/session_usage_logger.cc	2010-09-20 02:11:07 +0000
@@ -0,0 +1,56 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "config.h"
+
+#include "plugin/performance_dictionary/dictionary.h"
+
+#include <drizzled/session.h>
+
+#include <sys/resource.h>
+
+using namespace drizzled;
+using namespace std;
+
+namespace performance_dictionary {
+
+bool SessionUsageLogger::pre(Session *session)
+{
+  session->setUsage(true);
+
+  return false;
+}
+
+bool SessionUsageLogger::post(Session *session)
+{
+  QueryUsage *usage_cache= dynamic_cast<QueryUsage *>(session->getProperty("query_usage"));
+
+  if (not usage_cache)
+  {
+    usage_cache= new QueryUsage;
+    session->setProperty("query_usage", usage_cache);
+  }
+
+  usage_cache->push(session->getQueryString(), session->getUsage());
+
+  return false;
+}
+
+} /* namespace performance_dictionary */

=== added file 'plugin/performance_dictionary/session_usage_logger.h'
--- plugin/performance_dictionary/session_usage_logger.h	1970-01-01 00:00:00 +0000
+++ plugin/performance_dictionary/session_usage_logger.h	2010-09-20 02:11:07 +0000
@@ -0,0 +1,48 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_PERFORMANCE_DICTIONARY_SESSION_USAGE_LOGGER_H
+#define PLUGIN_PERFORMANCE_DICTIONARY_SESSION_USAGE_LOGGER_H
+
+#include <sys/time.h>
+#include <sys/resource.h>
+
+namespace performance_dictionary {
+
+class SessionUsageLogger: public drizzled::plugin::Logging
+{
+public:
+
+  SessionUsageLogger() :
+    drizzled::plugin::Logging("session_usage_logger")
+  {
+  }
+
+  ~SessionUsageLogger()
+  {
+  }
+
+  bool pre(drizzled::Session *);
+  bool post(drizzled::Session *);
+};
+
+} /* namespace performance_dictionary */
+
+#endif /* PLUGIN_PERFORMANCE_DICTIONARY_SESSION_USAGE_LOGGER_H */

=== added directory 'plugin/performance_dictionary/tests'
=== added directory 'plugin/performance_dictionary/tests/r'
=== added file 'plugin/performance_dictionary/tests/r/session_usage.result'
--- plugin/performance_dictionary/tests/r/session_usage.result	1970-01-01 00:00:00 +0000
+++ plugin/performance_dictionary/tests/r/session_usage.result	2010-09-20 02:11:07 +0000
@@ -0,0 +1,3 @@
+select count(*) from DATA_DICTIONARY.session_usage;
+count(*)
+0

=== added directory 'plugin/performance_dictionary/tests/t'
=== added file 'plugin/performance_dictionary/tests/t/master.opt'
--- plugin/performance_dictionary/tests/t/master.opt	1970-01-01 00:00:00 +0000
+++ plugin/performance_dictionary/tests/t/master.opt	2010-09-20 02:11:07 +0000
@@ -0,0 +1,1 @@
+--plugin-add=performance_dictionary

=== added file 'plugin/performance_dictionary/tests/t/session_usage.test'
--- plugin/performance_dictionary/tests/t/session_usage.test	1970-01-01 00:00:00 +0000
+++ plugin/performance_dictionary/tests/t/session_usage.test	2010-09-20 02:11:07 +0000
@@ -0,0 +1,2 @@
+#
+select count(*) from DATA_DICTIONARY.session_usage;

